@@ -1,7 +1,9 @@
 import warnings
 from dataclasses import dataclass
+from typing import Optional
 
 import pandas as pd
+import pandas.api.types as pd_types
 import plotly.express as px
 import plotly.graph_objs as go
 from scipy.stats import chisquare
@@ -19,15 +21,20 @@ class SRMCheckResult:
     chi_square: float
     p_value: float
     significant: bool
+    segment: Optional[str] = None
 
     def __repr__(self) -> str:
-        return f"variant: {self.variant}, control:treatment = {self.sample_size_c}:{self.sample_size}, p_value = {self.p_value}"
+        if self.segment:
+            return f"variant: {self.variant}, segment: {self.segment} control:treatment = {self.sample_size_c}:{self.sample_size}, p_value = {self.p_value}"
+        else:
+            return f"variant: {self.variant} control:treatment = {self.sample_size_c}:{self.sample_size}, p_value = {self.p_value}"
 
 
 class ABTestEvaluator(BaseEvaluator):
     def __init__(self) -> None:
         super().__init__()
         self.variant_col: str = ""
+        self.segment_col: str = ""
 
     # ignore mypy error temporary, because the "Self" type support on mypy is ongoing. https://github.com/python/mypy/pull/11666
     def evaluate(
@@ -36,6 +43,7 @@ class ABTestEvaluator(BaseEvaluator):
         unit_col: str,
         metrics: list[str],
         variant_col: str = "variant",
+        segment_col: Optional[str] = None,
     ) -> Self:  # type: ignore
         """calculate stats of A/B test and cache it into the class variable.
         At first, it only assumes metrics can handle by Welch's t-test.
@@ -47,11 +55,15 @@ class ABTestEvaluator(BaseEvaluator):
             The data should have been aggregated by the randomization unit.
         unit_col : str
             A column name stores the randomization unit. something like user_id, session_id, ...
+        metrics : list[str]
+            Columns stores metrics you want to evaluate.
         variant_col : str
             A column name stores the variant assignment.
             The control variant should have value 1.
-        metrics : list[str]
-            Columns stores metrics you want to evaluate.
+        segment_col : Optional[str]
+            A column name stores 'segment' you want to break down in the analysis.
+            e.g., light users, heavy users, new registers, ...
+            When the specified column in dataframe stores numerical values, it automatically binning the values.
 
         Returns
         -------
@@ -59,7 +71,20 @@ class ABTestEvaluator(BaseEvaluator):
             Evaluator storing statistics calculated.
         """
         self._validate_passed_data(data, unit_col, metrics)
-        self.stats = t_test(data, unit_col, variant_col, metrics)
+        if segment_col:
+            segment = data[segment_col]
+            if pd_types.is_numeric_dtype(segment) and not pd_types.is_bool_dtype(segment):
+                segment = pd.qcut(x=segment, q=5)
+
+            stats = pd.DataFrame()
+            for s in segment.unique():
+                partial_stats = t_test(data.loc[segment == s], unit_col, variant_col, metrics)
+                partial_stats[segment_col] = s
+                stats = pd.concat([stats, partial_stats])
+            self.stats = stats
+            self.segment_col = segment_col
+        else:
+            self.stats = t_test(data, unit_col, variant_col, metrics)
         self.variant_col = variant_col
         return self
 
@@ -137,6 +162,9 @@ class ABTestEvaluator(BaseEvaluator):
         }
         if display_ci:
             viz_options["error_x"] = f"{diff_type}_ci_width"
+        if len(self.segment_col) > 0:
+            viz_options["facet_row"] = self.segment_col
+            viz_options["height"] = stats[self.segment_col].nunique() * 200
 
         srm_check_results = self._diagnose_srm()
         for result in srm_check_results:
@@ -179,7 +207,10 @@ class ABTestEvaluator(BaseEvaluator):
         list[SRMCheckResult]
         """
         self._validate_evaluate_executed()
-        stats_subset = self.stats[["variant", "count", "count_c"]].drop_duplicates()
+        if self.segment_col:
+            stats_subset = self.stats[["variant", f"{self.segment_col}", "count", "count_c"]].drop_duplicates()
+        else:
+            stats_subset = self.stats[["variant", "count", "count_c"]].drop_duplicates()
         chi2q, p_value = chisquare(f_obs=stats_subset[["count", "count_c"]], axis=1)
         stats_subset["chi_square"] = chi2q
         stats_subset["p_value"] = p_value
@@ -196,5 +227,7 @@ class ABTestEvaluator(BaseEvaluator):
                 p_value=row["p_value"],
                 significant=row["significant"],
             )
+            if self.segment_col:
+                result.segment = row[f"{self.segment_col}"]
             results.append(result)
         return results
